# Editor Workflow Tests

Testing in this area should revolve primarily around UI/UX touchpoints with basic Editor functionality.

## Special Requirements 
Multiple Monitors (Preferably with different resolutions/Windows scaling/DPI settings)

## Project Requirements
Any, though AutomatedTesting makes life easier

## Common Issues to Watch For

Test guidance will sometimes note specific issues to watch for. The common issues below should be watched for through all testing, even if unrelated to the current workflow being tested.
1. Console log errors/warnings/spam
2. Asserts

## Documents and Common Terms
* [Gems in Open 3D Engine](https://www.o3de.org/docs/user-guide/gems/)
* [Adding and Removing Gems in a Project](https://www.o3de.org/docs/user-guide/project-config/add-remove-gems/)

## Supported Graphical Configuration Variants

### Resolutions

* 1080p (1920x1080)
* 4k (3840 x 2160)
* WQD (3440x1440)
* WQD+ (3840×1600)

### Scaling

* 100
* 125
* 150
* 175
* 200

## Workflow 1: Editor launch, Level CRUD, Entity/Component CRUD

**Product:** 
Saved and exported level with an interesting scene

**Suggested Time Box:** 
30 mins

| Workflow                                                               | Requests                                                                                                                                                                                                                      | Things to Watch For                                                                                                                                                        |
|------------------------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Launch Editor                                                          |                                                                                                                                                                                                                               | <ul><li>Excessive load times.</li><li>Inaccurate splash screen/start screen art/text.</li><li>Asset Processor launch.</li></ul>                                            |
| Create a new level/Open an existing level                              |                                                                                                                                                                                                                               | <ul><li>Excessive load times.</li></ul>                                                                                                                                    |
| Create several new entities                                            | <ol><li>Create some in Entity Outliner/some in Viewport.</li><li>Try duplicating/copy pasting entities to create new copies.</li><li>Drag in prefabs from Asset Browser.</li><li>Instantiate prefabs from context menu.</li><li>Undo/Redo.</li></ol>| <ul><li>Entity names.</li></ul>                                                                                                                      |
| Add/edit various components to entities to create an interesting scene | <ol><li>As components are added to entities, position entities around Viewport using Transform/Rotate/Scale controls with snapping both enabled/disabled.</li><li>Make changes via Transform component as well.</li><li>Undo/Redo.</li></ol>| <ul><li>Component property control issues (Sliders/dropdowns/text boxes).</li><li>Component dependencies.</li><li>Rendering changes reflected in Viewport.</li><li>Viewport controls.</li></ul>|
| Enter Game Mode                                                        |                                                                                                                                                                                                                               |                                                                                                                                                                            |
| Save/Export level                                                      |                                                                                                                                                                                                                               | <ul><li>Asset Processor errors with level processing.</li></ul>                                                                                                            |
| Resolution and Scaling												 | <ol><li>Launch the Editor and load any level.</li><li>Choose from the [Supported Graphical Configuration Variants](#supported-graphical-configuration-variants) three pairs of [Resolution](#resolutions) and [Scaling](#scaling).</li><li>Check the pairs configuration on your system.</li><li>Check each configuration with windowed and fullscreen Editor.</li></ol>| <ul><li>Restarting the editor after toggling full screen/windowed mode.</li><li>During full regression check all listed [Resolution](#resolutions) and [Scaling](#scaling) pairs.</li></ul>|
---

## Workflow 2: UI Layout Customization, Docking/Window Management

**Product:** 
Saved Editor UI Layout

**Suggested Time Box:** 
10-15 mins

| Workflow                                                                                                         | Requests                                                                                                                        | Things to Watch For                                                  |
|------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------|
| Reset to Default Editor Layout                                                                                   |                                                                                                                                 | <ul><li>Incorrectly sized/hidden windows.</li></ul>                  |
| Open several commonly used tools                                                                                 | <ol><li>Use a mixture of hotkeys/tools menu.</li></ol>                                                                          | <ul><li>Tools failing to open/opening offscreen.</li></ul>           |
| Move some tools to secondary monitor                                                                             |                                                                                                                                 | <ul><li>Scaling issues/windows popping offscreen/hiding.</li></ul>   |
| Dock some tools together and leave some windows free floating/snapped to other windows to create a custom layout | <ol><li>Dock side-by-side/create a tabbed group.</li><li>Use both monitors.</li></ol>                                           | <ul><li>Unusable tools.</li><li>Hidden windows/panes.</li><li>Scaling issues.</li></ul>|
| Name and save the custom layout                                                                                  |                                                                                                                                 |                                                                      |
| Load back and forth between a few different layouts                                                              | <ol><li>Verify layouts load successfully with elements on both monitors.</li><li>Relaunch Editor while testing to ensure proper saves.</li></ol>| <ul><li>Missing tools/unsaved/unloaded changes.</li></ul>|
---

## Workflow 3: Prefab Editing Focus Mode

**Product:** 
No tangible product, this workflow covers updating/manipulating tangible products, but doesn't itself generate any

**Suggested Time Box:** 
5-10 mins

**Setup:**
Create a Prefab with at least 2 entities underneath, each having a Mesh component with an assigned asset.

| Workflow                                                                                                                                                                                 | Requests                                                                                                                                 | Things to Watch For                                                                                                                                                                                                                                                            |
|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| With the prefab instance closed for editing click the Mesh entities making up the prefab in the Viewport                                                                                 |                                                                                                                                          | <ul><li>Clicking the entities that are a part of the prefab in the Viewport should only select the prefab root entity.</li><li>It should not be possible to select the entities inside a prefab while it is not being edited, in any way, either through Viewport or Entity Outliner.</li></ul>|
| Enter Focus Mode to edit a prefab                                                                                                                                                        | <ol><li>Utilize both double-clicking and `right-click` context menu's Edit Prefab option.</li></ol>                                      | <ul><li>Entity Outliner should update and display the contents of the prefab being edited.</li><li>Color of the container should change to a lighter blue.</li></ul>                                                                                                           |
| With the prefab instance opened for editing click entities outside of the prefab instance                                                                                                | <ol><li>Try from within both the Viewport and the Entity Outliner.</li><li>Try to toggle Hide/Lock for entities outside of the prefab instance.</li></ol>| <ul><li>There should be clear a visual indication that entities outside of the focus tree are not selectable (WIP).</li></ul>                                                                                                                                  |
| With the prefab instance opened for editing verify the breadcrumb widget at the upper-left of the Viewport displays the currently focused prefab, and the path to it from the level root | <ol><li>Add additional entities to the prefab hierarchy and click through the breadcrumb widget to ensure the mode changes as expected.</li></ol>|                                                                                                                                                                                                                                                                        |
---

## Workflow 4: Read-Only Entities

**Product:** 
No tangible product, this workflow covers updating/manipulating tangible products, but doesn't itself generate any

**Suggested Time Box:** 
5-10 mins

**Setup:**
Instantiate a Procedural Prefab (Example Procedural Prefabs in AutomatedTesting: _Objects\\sphere\_5lods\_fbx.procprefab, multiple\_mesh\_one\_material\\multiple\_mesh\_one\_material\_fbx.procprefab_

| Workflow                                                                                                                                              | Requests                                                                                                                                                                                                                                                                                                                              | Things to Watch For                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
|-------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Instantiate a Procedural Prefab                                                                                                                       | <ol><li>Try via `right-click` context menu.</li><li>Try via drag-and-drop from the Asset Browser. (Not Yet Implemented)</li></ol>                                                                                                                                                                                                     | <ul><li>Unusual delays in instantiation.</li><li>Console errors/asserts.</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Entity Outliner: Procedural Prefab container is distinct from normal Prefabs (currently Purple container color as opposed to Blue for normal prefabs) | <ol><li>Enter Focus Mode on the Procedural Prefab container:</li></ol>                                                                                                                                                                                                                                                                | <ul><li>Procedural Prefab container is distinct from normal Prefabs. (currently Purple container color as opposed to Blue for normal prefabs)</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     |
| Entity Outliner: Mouseover container entity                                                                                                           |                                                                                                                                                                                                                                                                                                                                       | <ul><li>Tooltip reads, "Double-click to inspect".</li><li>Container icon is unique with an eye icon in place of the normal pencil icon.</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| Entity Outliner: Enter Focus Mode on the Procedural Prefab container                                                                                  | <ol><li>Test via `right-click` context menu and double-clicking the container entity.</li></ol>                                                                                                                                                                                                                                       | <ul><li>Hide/Lock functionality is disabled on the container and all child read-only entities.</li><li>Read-only entities have unique icon with a lock icon over the normal box icon.</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             |
| Entity Outliner: While in Focus Mode, attempt CRUD operations on read-only entities via context menu                                                  | <ol><li>Try reparenting entities.</li><li>Try deleting entities.</li><li>Try Hiding/Locking entities.</li><li>Try parenting non-read-only entities to read-only entities and via the Transform component Parent entity reference in the Entity Inspector.</li><li>Try dragging an asset/prefab from the Asset Browser over the read-only entities.</li></ol>| <ul><li>Only options available to read-only entities are "Open pinned Inspector" and "Find in viewport".</li><li>Read-only entities and their children cannot be reparented or deleted.</li><li>Non read-only entities cannot be parented to read-only entities via UI operations.</li><li>Selecting read-only entities in the Outliner selects the corresponding entity in the Viewport, but no manipulator gizmos are usable.</li><li>Selecting read-only entities in the Outliner displays the component information in the Entity Inspector:</li><ul><li>Components should be greyed out.</li><li>Name field is not editable.</li><li>No new components can be added.</li><li>Existing components cannot be removed.</li><li>Components can be copied</li></ul><li>Dragging an asset/prefab from the Asset Browser over the read-only entities is not allowed.</li></ul>|
| Entity Outliner: Perform right-click context menu operations on the Procedural Prefab container                                                       | <ol><li>Utilize hotkeys as well to perform actions where possible.(e.g. Delete and Duplicate)</li></ol>                                                                                                                                                                                                                               | <ul><li>Instantiate Prefab/Procedural Prefab is not allowed.</li><li>"Save Prefab to file" should result in an appropriate error message as changes cannot be saved.</li><li>"Create Prefab" should create a new normal prefab with the Procedural Prefab nested inside.</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Viewport: Click the container entity in the Viewport should enable manipulator gizmos                                                                 |                                                                                                                                                                                                                                                                                                                                       | <ul><li>Manipulator gizmos should be available for the container when not in Focus Mode.</li></ul>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          |
| Viewport: Double-click the container entity in the Viewport to enter Focus Mode                                                                       |                                                                                                                                                                                                                                                                                                                                       | <ul><li>Focus Mode is entered.</li><li>No manipulator gizmos are available on the container entity or child read-only entities.</li><li>`Right-click` context menus are limited to "Open pinned Inspector" and "Find in viewport".</li><li>Selecting read-only entities in the Outliner displays the component information in the Entity Inspector:</li><ul><li>Components should be greyed out.</li><li>Name field is not editable.</li><li>No new components can be added.</li><li>Existing components cannot be removed.</li><li>Components can be copied.</li></ul><li>Dragging an asset/prefab from the Asset Browser into the Viewport while a Procedural Prefab is focused is not allowed.</li></ul>                                                                                                                                 |
---

## Workflow 5: Gem dependencies

**Product:** 
A project with the White Box and the Script Canvas gems activated.

​
**Suggested Time Box:** 15 mins

| Workflow                                                               | Requests																																																																						   | Things to Watch For                                                                                                                                                          |
|------------------------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| White Box gem															 | <ol><li>Create an entity and add the White Box component to it.</li><li>Save the level and turn the Editor off.</li><li>In the Project Manager select the project's hamburger menu and select configure gems.</li><li>Search for the White Box gem, deactivate it and save.</li><li>Rebuild the project, launch it and open the previously created level.</li><li>Search for the White Box component in the "Add component" dropdown list.</li></ol>| <ul><li>Correct and informative error messages.</li><li>Component correctly deactivated and not selectable.</li></ul>|
| Script Canvas	gem														 | <ol><li>Verify that the Script Canvas tool is listed in the Tools menu.</li><li>Create an entity and add the Script Canvas component to it.</li><li>Save the project and turn it off.</li><li>In the Project Manager select the project's hamburger menu and select configure gems.</li><li>Search for the Script Canvas gem, deactivate it and save. (Confirm the popup informing of no longer needed dependencies)</li><li>Rebuild the project and open it.</li><li>Verify that the Script Canvas in not listed in the Tools menu.</li></ol>| <ul><li>Correct and informative error messages.</li><li>Prompt about no longer needed dependent gems.</li><li>Component correctly deactivated and not selectable.</li><li>Script Canvas removed from the Tools menu.</li></ul>|
---

## Workflow 6: Asset Browser

**Product:** No tangible product, this workflow covers UX for the Asset Browser, allowing managing and adding assets

**Suggested Time Box:** 5-10 mins

| Workflow                    | Requests                                                                                                                                                                                                                                                                                                | Things to Watch For                                                                                                                                                                                                                                                                                                                                                       |
|-----------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Drag and drop files         | <ol><li>Drag and drop asset/prefabs into the Viewport</li><li>Test with List/Thumbnail/Table views</li></ol>                                                                                                                                                                                            | <ul><li>Dragging an asset/prefab from the Asset Browser into the Viewport should utilize mouse cursor placement when dropped.</li></ul>                                                                                                                                                                                                                                   |
| Move files and folders      | <ol><li>Use Right-click context menu > Move to</li><li>Use drag/drop operations</li><li>Test with List/Thumbnail/Table views</li></ol>                                                                                                                                                                  | <ul><li>Verify 'Pick folder to move to' modal appears</li></ul>                                                                                                                                                                                                                                                                                                           |
| Duplicate files and folders | <ol><li>Single files selected</li><li>Multiple selected files (at once)</li><li>Various file types</li><li>Utilize hotkeys as well to perform actions where possible</li><li>Test with List/Thumbnail/Table views</li></ol>                                                                             | <ul><li>Selecting single or multiple files should be reflected in the context menu</li></ul>                                                                                                                                                                                                                                                                              |
| Edit files                  | <ol><li>Double click selected files</li><li>Right-click context menu > Open with associated application...</li><li>Open various file types in various programs</li><li>Utilize hotkeys as well to perform actions where possible</li><li>Test with List/Thumbnail/Table views</li></ol>                 | <ul><li>Double clicking some file types will open in Editor by default, while others will, initially, open a programs list modal to edit the file in.</li><li>Selecting a program from the modal will consistently associate that file type to that program.</li><li>Right-Click context menu can be used to change the program associated with that file type.</li></ul> |
| Rename files and folders    | <ol><li>Right-click context menu > Rename</li><li>Utilize hotkeys as well to perform actions where possible</li><li>Test with List/Thumbnail/Table views</li></ol>                                                                                                                                      |                                                                                                                                                                                                                                                                                                                                                                           |
| Folder management           | <ol><li>Add folders: <ul><li>Right-click context menu > Create > Folder</li><li>Add via the + button</li></ul><li>Remove folders:</li><ul><li>Via right-click context menu<li>Via hotkey</li></ul><li>Test with List/Thumbnail/Table views</li></ol>                                                    |                                                                                                                                                                                                                                                                                                                                                                           |
| Delete files                | <ol><li>Delete a single file</li><li>Delete multiple files at once</li><li>Delete various file types</li><li>Utilize hotkeys as well as right-click context menu</li><li>Test with List/Thumbnail/Table views</li></ol>                                                                                 | <ul><li>Selecting single or multiple files should be reflected in the context menu</li></ul>                                                                                                                                                                                                                                                                              |
| Add new assets              | <ol><li>Right-click context menu > Create > \[Asset Type\]:<ul><li>ScriptCanvas Graph</li><li>Material</li><li>Material Graph</li><li>AnimGraph</li><li>MotionSet</li><li>Lua Script</li><li>Lua Component Script</li></ul><li>Test with List/Thumbnail/Table views</li><li>Test via + button</li></ol> | <ul><li>New asset should appear in the currently selected directory</li><li>New asset should be selected/prompt for a rename</li><li>Material/Material graph prompt using unique dialog to select type/path:</li><ul><li>Path should be filled in with current directory</li><li>Renaming completed via Windows Explorer dialog</li></ul>                                 |
| Asset Browser Inspector     | <ol><li>Open tool via Tools menu</li><li>Dock/Undock/Resize tool pane</li><li>Validate asset details appear when in List/Thumbnail/Table views</li></ol>                                                                                                                                                |                                                                                                                                                                                                                                                                                                                                                                           |
| Directory breadcrumbs       | <ol><li>Navigate to lower level directories via the breadcrumb links</li><li>Copy/paste directory address into breadcrumb box</li><li>Type directory address into breadcrumb box</li></ol>                                                                                                              |                                                                                                                                                                                                                                                                                                                                                                           |
| Back/Forward Navigation     | <ol><li>Navigate to various directories via the Back/Forward navigation buttons</li></ol>                                                                                                                                                                                                               | <ul><li>Behaves like a web browser, not for navigating up/down breadcrumb addresses</li></ul>                                                                                                                                                                                                                                                                             |
| Search/Filtering            | <ol><li>Filter to specific asset types in List/Thumbnail/Table views</li><li>Utilize text search in List/Thumbnail/Table views</li><li>Toggle Hide Engine Folders/Hide Unusable Product Assets</li></ol>                                                                                                |                                                                                                                                                                                                                                                                                                                                                                           |

## Workflow 7: DPE CVar Editor

**Product:** No tangible product, this workflow covers customizing console variables for use in the Editor

**Suggested Time Box:** 10-15 mins

| Workflow                   | Requests                                                                                                                                                                                                                                                                                                                                                                                              | Things to Watch For                                                                                                                                                                                                                                                                                      |
|----------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Open/Close the CVar Editor | <ol><li>Open via Tools > Other menu</li><li>Open via lower-left Console ( x ) button</li><li>Close via window ( x ) button</li><li>Close via title bar tab ( x ) button</li><li>Close via loading a saved layout without the CVar Editor active</li></ol>                                                                                                                                             | <ul><li>DPE Asserts</li><li>Console warning spam</li><li>Excessive load times</li></ul>                                                                                                                                                                                                                  |
| Search                     | <ol><li>Enter text to filter results</li><li>Clear search:</li><ul><li>Delete/Backspace</li><li>Clicking the X in the search bar</li><li>Closing/reopening the CVar Editor</li></ul><li>Search for values (e.g. True/False, specific ints/floats, etc.)</li></ol>                                                                                                                                     | <ul><li>Hitches during search filtering</li><li>Incorrect results</li><li>Searches should filter to valid results with matches in CVar names, tooltips, or values (ex. Searching for "true" will bring up all variables currently flagged to true, or with "true" in the tooltip or CVar name)</li></ul> |
| Set CVars                  | <ol><li>Set a value for each type of CVar:</li><ul><li>boolean</li><li>string</li><li>int</li><li>float</li><li>vec2</li><li>color</ul><li>Set values both via the CVar Editor itself, and via the console</li><li>Search a CVar name in the Console to see the currently set value</li><li>Increment/decrement values via the <> buttons</li><li>Relaunch the Editor after changing values</li></ol> | <ul><li>Incorrect values set</li><li>Mismatches in reported values between the Console and the CVar Editor</li><li>Warnings/Asserts</li><li>Hitches during value setting</li><li>Values are not saved between Editor launches unless set elsewhere (e.g. game.cfg)</li></ul>                             |