# Landscape Canvas Workflow Tests

Testing in this area should revolve primarily around Landscape Canvas graph configuration to create an interesting instance spawner setup

## Project Requirements
AutomatedTesting project, or another project with Atom, all Vegetation system, and Landscape Canvas Gems enabled

## Common Issues to Watch For
Test guidance will sometimes note specific issues to watch for. The common issues below should be watched for through all testing, even if unrelated to the current workflow being tested.
1. Console log errors/warnings/spam
2. Asserts

## Helpful Documentation

1. https://www.o3de.org/docs/user-guide/gems/reference/environment/
2. https://www.o3de.org/docs/user-guide/gems/reference/environment/landscape-canvas/
3. https://www.o3de.org/docs/user-guide/gems/reference/environment/surface-data/

### Workflow 1: Construction/configuration of an instance spawner via Landscape Canvas

**Product:** Prefab of constructed instance spawner

**Suggested Time Box:** 30 mins

| Workflow                                                                                           | Requests                                                                                                                                                                                                                                                                                                                                                                                                        | Things to Watch For                                                                              |
|----------------------------------------------------------------------------------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------|
| Construct several different planting surfaces (This must be performed outside of Landscape Canvas) | *   Use a mixture of emitter types using different surface tags<br>*   Create custom surface tags in Asset Editor                                                                                                                                                                                                                                                                                               | *   Component dependency issues<br>*   Surface Tag availability                                  |
| Create one/several new Landscape Canvas graphs                                                     | *   Create graphs via both manually adding Landscape Canvas component to an entity and via Landscape Canvas File menu                                                                                                                                                                                                                                                                                           | *   Automatic entity creation<br>*   Graph area updates                                          |
| Create one/several instance spawner area setups                                                    | *   Use multiple descriptors on a single spawner<br>*   Use varying spawner/blocker/blender areas<br>*   Use a mixture of Empty/Prefab spawner types<br>*   Configure descriptors to plant on a variety of previously created surfaces<br>*   Undo/Redo - Focused on node add/remove and connection add/remove<br>*   Make node connections manually in graph area and with entity targeting via Node Inspector | *   Node display issues<br>*   Connection display issues<br>*   Component dependency issues      |
| Organize/arrange nodes in graph area                                                               | *   Use Comment/Group nodes to organize/label<br>*   Navigate using large complex graphs (Bookmarks can help!)                                                                                                                                                                                                                                                                                                  | *   Node display issues<br>*   Connection display issues<br>*   Navigation issues                |
| Close/Reopen graph                                                                                 |                                                                                                                                                                                                                                                                                                                                                                                                                 | *   Node display issues<br>*   Connection display issues<br>*   Graph organization/layout issues |
| Save constructed surfaces + instance spawner areas constructed in LandscapeCanvas to a prefab      |                                                                                                                                                                                                                                                                                                                                                                                                                 | *   Missing components<br>*   Configuration changes                                              |
| Instantiate a copy of saved prefab                                                                 |                                                                                                                                                                                                                                                                                                                                                                                                                 | *   Missing components<br>*   Configuration changes                                              |

### Workflow 2: Gradient Baker Node

**Product:** Optimized Landscape Canvas graph

**Suggested Time Box:** 10 mins

| Workflow                                                                  | Requests                                                                                                                                                                                                                                                    | Things to Watch For                                                                                      |
|---------------------------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------|
| Create or open Landscape Canvas graph                                     | *   Create or open a Landscape Canvas graph containing the following:<br>    *   Shape Component<br>    *   Various connected Gradient components/nodes                                                                                                     | *   Automatic entity creation<br>*   Graph area updates                                                  |
| Add a Gradient Baker node to the graph                                    | *   Add the Gradient Baker node to your graph both via manually adding the Gradient Baker component to an entity and via Landscape Canvas File menu<br>*   Connect the Gradient Baker node to the Shape component and to the end of the Gradient node chain | *   Node display issues<br>*   Component dependency issues                                               |
| Save/Bake the image                                                       | *   In the Gradient Baker node:<br>    *   Select an Output Path using the folder icon button and name newly baked image<br>    *   Select Bake Image                                                                                                       | *   Component dependency issues<br>*   Preview image update issues<br>*   Output image not being created |
| Connect the newly baked node directly to the Shape component              | *   Connect the newly baked node directly to the Shape component<br>*   Select and disable the various connected Gradient components/nodes                                                                                                                  | *   Node display issues<br>*   Connection display issues                                                 |
| Close/Reopen graph                                                        |                                                                                                                                                                                                                                                             | *   Node display issues<br>*   Connection display issues<br>*   Graph organization/layout issues         |
| Save entity containing Gradient Baker node in LandscapeCanvas to a prefab |                                                                                                                                                                                                                                                             | *   Missing components<br>*   Configuration changes                                                      |
| Instantiate a copy of saved prefab                                        |                                                                                                                                                                                                                                                             | *   Missing components<br>*   Configuration changes                                                      |
