# Script Canvas Workflow Tests

Testing in this area should focus on the functionality of Script Canvas editor.

## Common Issues to Watch For

Test guidance will sometimes note specific issues to watch for. The common issues below should be watched for through all testing, even if unrelated to the current workflow being tested.

* Asset processor errors when loading/saving graph files
* Warnings or assets that appear in the graph validation log
* Unresponsive UI elements

### Platforms

* Windows
* Linux

### Documents and Common Terms

* [O3DE Documentation: Script Canvas](https://www.o3de.org/docs/user-guide/scripting/script-canvas/)  
* [O3DE Documentation: Script Canvas Editor Interface Guide](https://www.o3de.org/docs/user-guide/scripting/script-canvas/get-started/editor-interface/)  
* [O3DE Documentation: User Functions](https://www.o3de.org/docs/user-guide/scripting/script-canvas/editor-reference/functions/)

**Script Canvas Editor**: Editing tool separate from O3DE's Editor used to create and modify script canvas graph files.

**Script Canvas Graph**: A collection of node objects that can be modified and saved to a file.

**Script Canvas Editor Basic Tools**: The 5 default tools opened and anchored to the main Script Canvas editor window; Node Palette, Variable Manager, Node Inspector, Minimap, Graph Validation.

**Script Canvas Component**: A component that can be searched and attached to an entity in O3DE editor.

**User Function**: A type of Script Canvas graph that has 2 special nodes: Entrance & Exit nodelings. User Functions are intended to give users a way to generate re-usable graph logic similar to a function in a standard programming language. When a valid User Function graph is created and saved it will appear in the Node Inspector under the 'User Functions' category.

### Area: Opening and configuring the Script Canvas Editor

**Project Requirements** 

Any project can be used, the Script Canvas gem and its gem dependencies are required.

**Product:** Visible and responsive Script Canvas editor.

**Suggested Time Box:** 30 minutes

| Workflow                                                           | Requests                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Things to Watch For                                                                                                                                                                                                                                                                                                |
|--------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Script Canvas editor can be launched and configured                   | <ol><li>Open Script Canvas editor by multiple ways including:<ul><li>Tools menu</li><li>Script Canvas button in components</li><li>Keyboard shortcut ( a shortcut can be assigned via Edit > Editor Settings > Keyboard Customization > Customize Keyboard)</li></ul></li><li>Perform resize, maximize, minimize undock, dock actions with Script Canvas editor and its panes and drag them around the Editor.</li><li>Close and reopen the Script Canvas editor.</li></ol> | <ul><li>Script Canvas editor not launching quickly or not launching at all.</li><li>Script Canvas Editor not retaining configuration settings on close/relaunch.</li><li>Script Canvas panes not rendering, having rendering issues after being altered.</li><li>Not being able to resize, dock or drag panes borders.</li> |
| Script Canvas window pane renders a script canvas graph | <ol><li>Create a new script in Script Canvas editor by multiple ways including:<ul><li>Tools menu File > New Script</li><li>Using Ctrl+N shortcut</li><li>By dragging a node from the Node Pallete into an empty graph window</li></ul></li><li>Close the created script and save it when prompted.</li><li>Open the created script.</li><li>Create multiple new scripts in Script Canvas editor.</li></ol>                                                                                                                                                    | <ul><li>Modifying an existing file places am asterisk in the graph's tab.</li><li>An asterisk appears on the tab only if a change is made to the file.</li><li>User is always prompted to save a modified file when closing the tab.</li><li>Creating or opening multiple files generates multiple tabs in the main window.</li><li>Creating and having several graphs opened should not impact Script Canvas editor's performance.</li><li>A drop down menu for file selection appears when enough script canvas files are opened.</li></ul>                                             |
| Script Canvas editor tools can be opened and configured  | <ol><li>Open each of the Script Canvas editor tools.</li><li>Click and try to drag every UI item of the opened tool.</li></ol>                                                     | <ul><li>Script Canvas tools render properly.</li><li>Tools do not become unresponsive after configuring them.</li><li>UI elements in each of the tools respond when clicked or dragged.</li>                                                                                                                                                                   |
| Using basic Script Canvas graph controls | <ol><li>Create a new script canvas graph file, add a few nodes and perform every action from:<ul><li>View menu</li><li>Edit Menu</li><li>File menu</li></ul></li><li>Perform every control action located in Script Canvas graph window top left corner.</li></ol> | <ul><li>Every control action and action from Script Canvas menu bar perform corresponding actions.</li><li>Every control action and action from Script Canvas menu can be performed with associated keyboard shortcut.</li><li>Controls not responding or responding too slowly when used.</li><li>Controls gray out under appropriate circumstances.</li><li>Grayed out controls can not be interacted with.</li><li>A helpful tooltip is generated and rendered properly for each control.</li><li>Tooltips do not go away prematurely.</li><li>Script Canvas graph renders properly after Pan/Zoom actions.</li></ul>                                                     
| Opening a Script Canvas graph from older version of the engine            | <ol><li>Obtain a script canvas graph file from an earlier version of O3DE , put it into used project directory and open it with Script Canvas editor.</li><li>Following graph file [OldGraphTest.scriptcanvas](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/scripts/OldGraphTest.scriptcanvas) derived from older o3de development build can be used for this test.</li></ol> | <ul><li>Graph from the older version of the engine is processed by Asset Processor without errors and can be opened with Script Canvas editor.</li><li>If [OldGraphTest.scriptcanvas](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/scripts/OldGraphTest.scriptcanvas) was used and assigned to entity upon entering the game mode/simulation mode a message reading _Old Graph test successful_ should appear in the console.</li> |
| Opening a Script Canvas graph with deprecated nodes                   | <ol><li>Obtain a script canvas graph file with deprecated nodes, put it into used project directory and open it with Script Canvas editor.</li><li>Following graph file [ontick.scriptcanvas](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/scripts/ontick.scriptcanvas) derived from older o3de development build can be used for this test.</li></ol> | <ul><li>Graph with deprecated nodes is processed by the Asset Processor with errors reminding the user of the deprecated classes but the graph can still be opened with Script Canvas editor.</li><li>When the graph with deprecated nodes is opened in Script Canvas editor it runs validation check on itself and informs the user that listed nodes were updated in order for the graph to work.</li><li>When the graph is saved with proposed updates it is no longer displayed as processed with errors in the Asset Processor.</li><li>If [ontick.scriptcanvas](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/scripts/ontick.scriptcanvas) was used, updated and assigned to entity upon entering the game mode/simulation mode a message reading _OnTick Event:The Sphere position did not change from previous position_ should appear in the console.</li> |
---

### Area: Default Script Canvas Editor tools

**Project Requirements** 
<p>Script Canvas editor opened with the complete list of tools which are:

* Node Palette
* Node Inspector
* Variable Manager
* Minimap
* Graph Validation
* Debugging
* Bookmarks
* Command Line
* Interpreter

**Product:** 
Responsive Script Canvas editor with all its tools enabled. No Asset Processor warnings about invalid data or files, no warnings or errors in the Console Log.

**Suggested Time Box:** 
60 minutes

| Workflow                                                           | Requests                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Things to Watch For                                                                                                                                                                                                                                                                                                |
|--------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Use and observe minimap behavior                   | <ol><li>Open or create a graph with multiple nodes</li><li>Drag the minimap view around.</li><li>Switch between multiple graphs.</li></ol> | <ul><li>If the minimap does not render or update properly.</li><li>If dragging the minimap view does not cause the effect in main editor window and vice-versa.</li> |
| Create a variable node with Variable Manager | <ol><li>Use Variable Manager to create a variable of any type.</li><li>Modify the properties of the variable.</li><li>Drop the variable off the Variable Manager list onto a graph.</li><li>Drag the variable off the Variable Manager list onto a node's input/ouput field (same type).</li><li>Convert a node's input/ouput field to a reference and drag/select variables from the Variable Manager onto it.<ol>                                                                                                                                                    | <ul><li>If variable properties do not update when modified.</li><li>Variable Manager list refreshes and updates properly when variables are created, deleted or modified.</li><li>Node Inspector updates when variables are selected in the Variable Manager.</li></ul>                                             |
| View and Modify a node with Node Inspector  | <ol><li>Click a node on the graph.</li><li>Modify node values through Node Inspector.</li><li>Add and select different nodes and variables.</li><li>Select multiple nodes at once.</li></ol>                                                     | <ul><li>Fields and values of nodes are displayed and properly labeled in Node Inspector.</li><li>Changes made through Node Inspector are reflected on the node and vice-versa.</li><li>Selecting multiple nodes at once is reflected in Node Inspector.</li>                                                                                                                                                                   |
| Create a graph validation error and warning | <ol><li>Drag several nodes onto a fresh SC graph and do not connect them.</li><li>Run validation check on the graph.</li><li>Connect nodes and run validation.</li><li>Run the graph validation using dedicated button in Graph Validation pane (checkmark in circle).</li></ol> | <ul><li>If Highlighting & Dimming of failing nodes is not updated.</li><li>If warnings and errors are not removed when issue is resolved.</li><li>If conditions that produce an invalid graph do not trigger a warning or error.</li></ul>                                                     
| Use the Node Palette to find node types            | <ol><li>Search the Node Palette for a specific node type.</li><li>Use the scrolling view controls to browse nodes.</li><li>Drag a node off the palette onto a new SC graph.</li><li>Use right click context menu on a SC graph to search & browse for nodes.</li><li>Mouse over a node in the palette to display a tooltip.</li></ol> | <ul><li>Node palette renders properly when entering/deleting text in search field.</li><li> Node categories can be expanded and collapsed.</li> |
| Modify Global Preferences                  | <ol><li>Open Global Preferences and adjust the time required for certain actions to take place. (for instance Node Coupling, Drag Node Splicing).</li><li>Modify Styling Settings of connections.</li><li>Modify Edge Panning Settings.</li></ol> | <ul><li>If nodes are not visually distorted.</li><li> If modification of time requirement for certain actions to take place is not applied.</li> |
| Using File Menu                   | <ol><li>Create a new script.</li><li>Open an existing script.<li>Use Save script / save script as... functions to save the script.</li><li>Close script.</li><li>Use available keyboard shortcuts to perform above mentioned actions.</li></ol> | <ul><li>If creating a script does not return error in Console.</li><li>Recently saved/created scripts are displayed in "Open Recent" menu.</li><li>Scripts can be saved.</li><li>No errors are printed in the console after saving a script.</li><li>Closing unsaved script displays a message reminding of unsaved changes.</li> |
| Using Edit Menu                  | <ol><li>Undo and redo various actions.</li><li>Manage nodes, groups, notes via Edit Menu by performing following actions:<ul><li>Cut</li><li>Copy</li><li>Paste</li><li>Duplicate</li><li>Delete</li></ul></li><li>Take a screenshot of the graph.</li><li>Select all, inputs, outputs, connected, and clear selections.</li><li>Disable and enable selected nodes.</li><li>Align nodes in various directions.</li><li>Remove unused variables and nodes.</li><li>Use available keyboard shortcuts to perform above mentioned actions.</li></ol> | <ul><li> If nodes are not visually distorted.</li><li>Undoing or redoing actions does not cause an assert or errors to be printed in the console.</li><li> Copy-Pasted nodes do retain their modified properties.</li> |
| Using View Menu                  | <ol><li>Zoom graph in and out.</li><li>Show selection:<ul><li>individual node</li><li>multiple connected nodes</li><li>group</li><li> notes</li><li>entire graph</li></ul></li><li>Use "Go to" option to navigate to start and end of the chain (graph must have multiple nodes connected with each other).</li><li>Restore default layout of Script Canvas tool.</li><li>Use available keyboard shortcuts to actions mentioned above.</li></ol> | <ul><li>Show selection or show entire graph options are displaying graph accurately.</li><li>Default layout can be restored.</li> |
| Using Tools Menu                  | <ol><li>Open and close tools available in Tools Menu.</li><li>Use available keyboard shortcuts to open and close all tools.<li>Perform resize, maximize, minimize, undock and dock actions with all of Script Canvas editor tools and drag them around the Script Canvas editor.</li></ol> | <ul><li>Tools are opened after selecting them in Tools Menu.</li><li>Shortcuts can be used to open/close tools.</li><li>The user is able to resize, minimize, maximize, undock, dock or drag Script Canvas editor tools around the Script Canvas editor.</li> |
| Using Script Events [Preview] Menu          | <ul><li> This menu is soon to be deprecated and reworked, until its updated its ommited from testing. </li><li> [O3DE User Guide: Script Events](https://www.o3de.org/docs/user-guide/scripting/script-events/) can be followed for the current state of the functionality. |
---

### Area: Basic Script Canvas graph tests

Testing in this area verify basic functionality of script canvas graphs and more in-depth functionality of the default editor tools.

**Project Requirements** 
<p>Script Canvas editor opened with the complete list of tools which are:

* Node Palette
* Node Inspector
* Variable Manager
* Minimap
* Graph Validation
* Debugging
* Bookmarks
* Command Line
* Interpreter

**References**
<details><summary>Default Script Canvas editor layout with the default tools enabled and highlighted.</summary>
<p>

[![SC graph editor and default tools](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/images/SCeditor_default_layout.png)](https://github.com/o3de/sig-content/blob/main/testing-guidance/workflow-tests/scripting/images/SCeditor_default_layout.png)

</p>
</details>

**Product:** 
Script canvas graph files with basic input and output when run through game mode or interpreter.

**Suggested Time Box:**
75 minutes

| Workflow                                                           | Requests                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Things to Watch For                                                                                                                                                                                                                                                                                                |
|--------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Create and save a new script canvas graph with basic input and output                   | <ol><li>Use any method to create a blank script canvas file.</li><li>Use the node palette or right click context menu to add and connect multiple nodes.</li><li>Create a logic sequence that will occur on graph start or on the activation of an entity.</li><li>Use the Variable Manager to create one or more new variables.</li><li>Change a field on a node to a reference and use the newly created variable.</li><li>Click on a variable you created and use the Node Inspector to modify it. Change its name, default value and scope.</li><li> Run the game mode or load the graph in the Interpreter.</li></ol> | <ul><li>If saving the file causes asset processor errors or warnings.</li><li>If repeatedly running the game causes the graph to produce no results or inconsistent results.</li><li>If any console warnings or errors regarding Lua appear.</li> |
| Create a 2nd script canvas graph file and use a script event to make the two files communicate during gameplay | <ol><li>Use the O3DE Asset Editor to create a new script event.</li><li>Add a new event to the script event asset. Give it a unique and recognizable name  </li><li>Add input parameters to the event and save the asset.</li><li>Search the Script Canvas editor's node inspector for the new event asset. [![Filtering Node Palette for a script event. Dragging the node onto the graph](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/images/SCeditor_script_events_workflow.png)](https://github.com/o3de/sig-content/blob/main/testing-guidance/workflow-tests/scripting/images/SCeditor_script_events_workflow.png) </li><li>Create a Send and Receive signal nodes for the new event.</li><li>Configure two or more script canvas files to send and receive signals from each other.  <br>**Examples:**     <ul><li>  Attach a send signal to a timer node so every other second the receiving script changes a light's color.     </li><li>Send multiple different script events and a script canvas file that has different logic predecated on receiving each of the different signals.</li></ul></li></ol>                                                                                                                          | <ul><li>If there are any tool display issues/unusable portions of tool.</li><li>Saved event asset appears in the directory in which it was saved.</li><li>Events can be used to send and receive signals from two or more script canvas files which can be proven by debug output or working graph logic.</li></ul>                                             |
| Animated behavior and visual language of Script Canvas.  | <ol><li>Create and hover one node above another, so the nodes are partially transparent.![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCnodes_dimming_hover_over.png)</li><li>Hold Alt key and left click on node to instantly delete it, square particles will appear.![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCnode_delete_particles.png) </li><li>Insert a node between two connected nodes, the cursor need to point onto connection, which will change to dashed line. The node-nudging is enabled by default, the nodes will move aside to accommodate new node.<p>![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCnode_nudging.png) </li></ol>                                                     | <ul><li>If nodes are not dimming/partially transparent when hovering one node over another node.</li><li>If node is deleted without particles visual effect when using Alt + LMB delete method.</li><li>If nodes are not nudging aside to accommodate new node.</li><li>If nodes are not connected after inserting node between two connected nodes.</li>                                                                                                                                                                   |                                                 
| Nodes behavior           | <ol><li>Add nodes with various data value types to the graph and modify their values.</li><li>Attempt to connect two different value type pins.</li><li>Add input and operand to the node (e. g. Print/Add (+) nodes).</li><li>Hide/show unused slots on the node.</li><li>Convert value to reference and reference to value by clicking RMB on data pin.</li><li>Disable and enable nodes.</li><li>Insert long string into node string value field to extend the node.</li></ol> | <ul><li>If data values cannot be modified.</li><li>If nodes with long strings are visually distorted.</li><li>If disabled nodes are not grayed out.</li> |
| Connections behavior                  | <ol><li>Create a connection between nodes by dragging connection from output pin onto input pin.</li><li>Drag a connection over empty space in graph and release LMB, in context menu search for desired node to be created and connected to.<li>Connect two nodes by using Node Coupling feature, mentioned feature is enabled by default in Global Preferences. Hover one node over another and hold it above until nodes connect.</li><li>Delete connection by selecting the connection and pressing Del key.</li><li>Delete connection by dragging one node and shaking it, shaking de-splice feature is enabled by default.</li><li>Delete connection by holding Alt key and clicking LMB on connection.</li></ol> | <ul><li>If connection is not created.</li><li>If node is not created after selecting it from context menu.</li><li>If connection cannot be deleted.</li> |
---

### Area: Advanced Script Canvas graph Tests

Tests in this verify more advanced core features of script canvas.

**Project Requirements** 
<p>Script Canvas editor opened with the complete list of tools which are:

* Node Palette
* Node Inspector
* Variable Manager
* Minimap
* Graph Validation
* Debugging
* Bookmarks
* Command Line
* Interpreter

**References**
<details><summary>Main Script Canvas editor view with advanced control buttons highlighted.</summary>
<p>

[![SC graph view and basic controls](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/images/SCeditor_graphview_controls.png)](https://github.com/o3de/sig-content/blob/main/testing-guidance/workflow-tests/scripting/images/SCeditor_graphview_controls.png)

</p>
</details>

**Product:**
Organized and annotated Script canvas graph files that produce verifiable outputs when attached to entities and run through game mode or interpreter.

**Suggested Time Box:**
150 minutes

| Workflow                                                           | Requests                                                                                                                                                                                                                                                                                                                                                                                                                                                                       | Things to Watch For                                                                                                                                                                                                                                                                                                |
|--------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Creating a user function and calling its input/output from another graph                   |<ol><li>Use Script Canvas editor to create a graph file with enter/exit nodelings. (An instance of such graph and buttons used to create nodelings can be seen in the picture below) [![](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/images/SCeditor_basic_userFunction00.png)](https://github.com/o3de/sig-content/blob/main/testing-guidance/workflow-tests/scripting/images/SCeditor_basic_userFunction00.png) </li><li>Place complex logic or entity behavior between the enter/exit nodelings.</li><li>Give the graph file a unique recognizable name and save it.</li><li>Call the user function from another graph by finding it in the node palette or right click context menu. [![Calling a user function from another graph](https://github.com/o3de/sig-content/raw/main/testing-guidance/workflow-tests/scripting/images/SCeditor_basic_userFunction01.png)](https://github.com/o3de/sig-content/blob/main/testing-guidance/workflow-tests/scripting/images/SCeditor_basic_userFunction01.png)</li></ol> **Example Functions:** <ol><li>Two color parameters and returns a blended color.</li><li>Entity parameter and returns the number of child entities.</li><li>No parameters or output but triggers behavior. | <ul><li>Recursive functions are **NOT** supported at the moment.</li><li>Modifications to the user function file should require scripts calling the function to also be modified.</li> |
| Generate and modify translation data | <ol><li> Select any node in Script Canvas Node Pallete and right click it.</li><li>Select a _Generate Translation_ option.</li><li>Open the generated _.names_ file and modify in with any text editor by for instance changing the node name. </li><li> Save the file and drag another instance of the corresponding node into the Script Canvas graph. <ol>                                                                                                                                                    | <ul><li>Generating translation data does not cause major performance dips, however please note that generating translation data takes about 3-5 seconds.</li><li>Non-english characters render properly in SC graph.</li><li>Translation data modifications are reflected in Script Canvas editor.</li></ul>                                             |
| Use groupings  | <ol><li>Create groups of all available types grouping nodes of an existing graph.</li><li>Move the groups around in the Script Canvas editor.</li><li>Resize groups and grab more nodes into them.</li><li>Open Node Inspector and select one of the created groups.</li><li>Perform all available actions found in Node Inspector for selected group (Enabling as Bookmark, Collapsing Group, Changing all Font settings, etc...).</li><li>Double click the groups in graph to collapse/expand them.</li><li>Perform actions from group edit menu by right clicking on a group in Script Canvas graph window (Copy/Paste, Duplicate, Collapse, etc...).</li></ol>                                                     | <ul><li>If groupings are not rendering when placed on the graph.</li><li>If nodes are not "Sticking" to a grouping when size or position is changed.</li><li>Modifications to groups are maintained when graph is saved/loaded.</li><li>Groups can have their all corresponding menu and Node Inspector actions performed.</li><li>Different fonts can be assigned to groups with Font Settings.                                                                                                                                                                  |
| Use annotations | <ol><li>Create a comment on an existing graph.</li><li>Move the created comment around in the Script Canvas editor. </li><li>Input strings into comment input field.</li><li>Open Node Inspector and select the comment.</li><li>Perform all available actions found in Node Inspector for selected comment (Comment, Background Color, etc...). </li><li>Double click the comment in graph to enter its edit mode.</li><li>Perform actions from comment edit menu by right clicking on a comment in Script Canvas graph window (Cut, Copy/Paste, Duplicate, Delete, Edit comment, etc...).</li></ol>                                                     | <ul><li>Comments can be created on the graph and strings can be written in their input field.</li><li>Comments size scales depending on the length of written string.</li><li>Different fonts can be assigned to comments with Font Settings.</li><li>Comments can have their all corresponding menu and Node Inspector actions performed.  
| Use bookmarks           | <ol><li>Create a bookmark on an existing graph.</li><li> Move the created bookmark around in the Script Canvas editor.</li><li>Open Node Inspector and select the comment.</li><li>Perform all available actions found in Node Inspector for selected bookmark( Bookmark Name and Color). </li><li>Open Bookmarks tool and perform available actions:<ul><li>Double click bookmark to enter its name edit mode</li><li>Right click bookmark and select Go to Bookmark action</li><li>Right click bookmark and select Delete Bookmark action</li><li>Double click Shortcut section and select a number shortcut for a bookmark</li></ul></li><li>Select a Script Canvas graph window and click a shortcut number associated with one of the bookmarks.</li><li>Perform actions from group edit menu by right clicking on a group in Script Canvas graph window (Cut, Copy/Paste, Duplicate, etc...). </li></ol> | <ul><li>Bookmarks can be created on the graph and are assigned with random color and a shortcut when available.</li><li> Bookmarks can have their color and name changed via Node Inspector.</li><li>Bookmarks can be edited with Bookmarks tool.</li><li>Clicking a number shortcut associated with one of the bookmarks finds and focuses the Script Canvas graph view on the selected bookmark.</li> |
| Create and use presets                 | <ol><li>Create a comment and a group.</li><li>Change their font settings and colors with Node Inspector.</li><li>Right click them in Script Canvas Graph pane and select "Create Preset From" option.</li><li>Save the selected comment and a group as separate presets.</li><li>Create a new comment and a group.</li><li>Right click them in Script Canvas Graph pane and select "apply preset".</li><li>Apply created presets and default presets to groups and comments.</li><li>Open the Presets Editor tool.</li><li>Switch between Node Group and Comment Construct Types.</li><li>Set various presets as default.</li><li>Remove various presets.</li><li>Click the Reset Button.</li></ol> | <ul><li>Presets can be made out of different comments and groups.</li><li>Presets can be applied to comments and groups and retain their assigned values, fonts and colors.</li><li>Presets can be viewed and removed in Presets Editor tool.</li><li>Using the Reset option in Editor Tool, resets presets to default, removing newly created presets and readding removed default presets.</li> |
| Use the Debugging tool to generate script logs                  | <ol><li> Create a simple graph with On Graph Start -> Delay -> Print nodes.</li><li>Create an entity with Script Canvas component in the viewport and assign it with previously created graph.<li>Open the Debbuger tool in Script Canvas Editor and select the created debug targets from dropdown.</li><li>Enable and disable Auto Capture and Live Updates options.</li><li>Run the Script Canvas Debugger with Capture button.</li><li>Run the Script Canvas Debugger by entering Game Mode in the main Editor window.</li><li>Examine captured data, expand and collapse lines with dedicated buttons.</li></ol> | <ul><li>Created entity appears in Entities tab of Debugging tool.</li><li>Existing script files are displayed in Graphs tab of Debugging tool.</li> |
| Use Interpreter tool to execute script                 | <ol><li>Create a simple graph with On Graph Start -> Delay (5s) -> Print nodes.</li><li>Open the Interpreter tool in Script Canvas Editor and assign previously created script to it.<li>Click Start button to execute the script.</li><li>Click Stop button during script execution.</li></ol> | <ul><li> Interpreter tool can be assigned with scripts.</li><li>Scripts can be executed with Interpreter tool.</li><li>Execution of scripts by the Interpreter can be stopped.</li> |
| Create a script to handle inputs                  | <ol><li>Create an Input Bindings file in Asset Editor tool and configure it as shown in the picture below. ![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCcreate_input_bindings_file.png)</li><li>Create a script with Input Handler node, set Event Name to match previously created input bindings file (Print). ![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCinput_handle_script.png)<li>Create an entity with Script Canvas and Input components, assign previously created Input Bindings file and script to corresponding components.</li><li>Enter Game Mode and press space key to print message in console.</li><li>Assign various keys in Input Bindings file.</li></ol> | <ul><li>Various inputs can be assigned to Input Handlers.</li><li>Script Canvas receives and handles assigned inputs.</li> |
| Create a script for Audio playback                 | **Prerequisites:**</li><ul><li>Wwise project is setup.</li><li>Wwise Audio Engine Gem is enabled.</li></ul></ul> <ol><li>Create an entity with Script Canvas, Audio Trigger, and Audio Proxy components.</li><li>In Audio Trigger component assign Play trigger.<li>Create a script with Execute Trigger node that will execute after short delay and assign it to Script Canvas component. ![](https://github.com/o3de/sig-content/blob/944ec3c3d0a3420f78d408e07d9ad146ffeaa78f/testing-guidance/workflow-tests/scripting/images/SCaudio_script.png)</li><li> Enter Game Mode.</li></ol> | <ul><li>Audio triggers can be triggered by Script Canvas.</li><li>Audio output can be heard when triggers are playing.</li> |
| Using Command Line                   | <ol><li>Open the Command Line tool with tools menu and Ctrl+P shortcut.</li><li>Type **?** in the command line text bar.<li>Select a command.</li><li>Click the Enter key.</li><li>Type in working command directly into a text bar and click the Enter key.</li></ol> | <ul><li>Command Line Tool can be opened and input can be written into its text bar.</li><li>Commands can be searched for by typing **?** into the text bar.</li><li>Found commands can be entered and corresponding actions such as creating new nodes in Script Canvas graph are executed by the command.</li><li>  Commands can be manually typed into the text bar and executed.</li> |
---
