# UI Editor Workflow Tests

Testing in this area should revolve primarily around UI Canvas creation to create an interesting canvas for loading in Game Mode.

## Project Requirements
Any project with LyShine Gems enabled

## Common Issues to Watch For

Test guidance will sometimes note specific issues to watch for. The common issues below should be watched for through all testing, even if unrelated to the current workflow being tested.
1. Console log errors/warnings/spam
2. Asserts

### Workflow 1: Construction/configuration of a UI Canvas via UI Editor

**Product:** 
Saved .uicanvas file. Saved level that loads a .uicanvas file on entering Game Mode.

**Suggested Time Box:** 
30 minutes

| Workflow                                       | Requests                                                                                                                                                                                                                                                                                               | Things to Watch For                                                                                                                                                                               |
|------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Launch and configure UI Editor                 | *   Reposition/Resize panes<br>*   Dock/Undock panes and tool window                                                                                                                                                                                                                                   | *   Tool display issues/unusable portions of tool                                                                                                                                                 |
| Add elements to canvas area                    | *   Use a mixture of empty elements with components manually added and slices from Slice Browser or Slice Library<br>*   Add slices via context-menu and Asset Browser(ui assets can be found at Gems/UiBasics/Assets/UI/Slices/Library)                                                               | *   Rendering issues with visual/text elements<br>*   Hierarchy issues<br>*   Element names                                                                                                       |
| Update element properties and arrange elements | *   Adjust element component properties to create an interesting looking canvas<br>*   Arrange elements via Move/Rotate/Scale gizmos<br>*   Arrange elements via alignment buttons<br>*   Update Transform2D properties<br>*   Adjust/change fonts on text elements                                    | *   Rendering issues with visual/text elements<br>*   UX issues with arrangement of elements in the canvas area                                                                                   |
| Create new UI slices                           | *   Create new UI slices from empty configured elements<br>*   Create cascaded UI slices from multiple slices<br>*   Update properties and push changes to slices<br>*   Hover over elements in the Hierarchy to observe slice info<br>*   Edit slices on an open canvas as well as in a new slice tab | *   Error messages on pushing changes to slices that are confusing/inaccurate<br>*   Text display issues in Hierarchy windows with slice labeling/tooltips<br>*   AP errors with slice processing |
| Preview the canvas                             | *   Interact with the canvas on the Preview screen<br>*   Adjust Preview size                                                                                                                                                                                                                          | *   Rendering issues with visual/text elements<br>*   Interactivity issues (Buttons, Drop-downs, etc.)                                                                                            |
| Save the canvas                                | *   Save and reload the canvas                                                                                                                                                                                                                                                                         | *   AP errors with canvas<br>*   Dirty indicator on canvas names not clearing ( denoted by \* )<br>*   Canvas name inaccuracies in tab area                                                       |
| Open a few more canvases                       | *   Re-arrange tabbed canvases<br>*   Close a single canvas<br>*   Make changes on a canvas and Undo/Redo                                                                                                                                                                                              | *   Only active canvas is affected by changes (other than slice pushes)                                                                                                                           |
| Load the canvas at runtime in the Editor       | *   Load the canvas via the UI Canvas Asset Ref component<br>*   Load the canvas automatically and/or via a script (ADVANCED)                                                                                                                                                                          | *   Rendering issues with visual/text elements<br>*   Interactivity issues (Buttons, Drop-downs, etc.)                                                                                            |
| Save level                                     |                                                                                                                                                                                                                                                                                                        | *   Asset Processor errors with level processing                                                                                                                                                  |
| Run Game                                       | *   Press Ctrl + G to run game in the editor<br>*   Run the level in the game launcher                                                                                                                                                                                                                 | *   Rendering issues with visual/text elements<br>*   Interactivity issues (Buttons, Drop-downs, etc.)                                                                                            |

### Workflow 2: High DPI Scaling

**Product:** 
Working UI Editor viewport display and navigation with various scaling configurations.

**Suggested Time Box:**
10 minutes

| Workflow                                                   | Requests                                                                                                                                                                                                                                                      | Things to Watch For                                                                                                  |
|------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|----------------------------------------------------------------------------------------------------------------------|
| Launch and configure UI Editor on a high DPI display (4K+) | *   Configure Windows display settings<br>    *   Scaling<br>    *   Resolution<br>    *   Orientation                                                                                                                                                        | *   Rendering issues with visual/text elements<br>*   UI Canvas opens off-center<br>*   Tool window opens off-screen |
| Configure Panes                                            | *   Reposition/Resize panes<br>*   Dock/Undock panes and tool window                                                                                                                                                                                          | *   Tool display issues/unusable portions of tool<br>*   Interactivity issues                                        |
| Navigate UI Canvas                                         | *   Navigate UI Canvas area with mouse  <br>    *   Pan<br>    *   Zoom<br>*   Load/Rearrange multiple tabbed UI Canvases                                                                                                                                     | *   Rendering issues with visual/text elements<br>*   Interactivity issues                                           |
| Add elements to canvas area                                | *   Use a mixture of empty elements with components manually added and slices from Slice Browser or Slice Library<br>*   Add slices via context-menu and Asset Browser(ui assets can be found at Gems/UiBasics/Assets/UI/Slices/Library)                      | *   Rendering issues with visual/text elements<br>*   Interactivity issues                                           |
| Arrange elements                                           | *   Arrange elements using gizmos<br>    *   Selection<br>    *   Move<br>    *   Rotate<br>    *   Resize<br>    *   Anchor<br>*   Toggle rulers and guides  <br>    *   Pan the canvas with rulers toggled on<br>    *   Move cursor with guides toggled on | *   Rendering issues with visual/text elements  <br>   *   Interactivity issues                                      |
| Preview the canvas                                         | *   Interact with the canvas on the Preview screen<br>*   Drag resize window<br>*   Adjust Preview size                                                                                                                                                       | *   Rendering issues with visual/text elements<br>*   Interactivity issues (Buttons, Drop-downs, etc.)               |

### Workflow 3: UI Canvas on Mesh in level

https://www.o3de.org/docs/user-guide/interactivity/user-interface/canvases/placing-canvases-3d/  
https://www.o3de.org/docs/user-guide/components/reference/ui/canvas-proxy-ref/

**Product:**
Saved .uicanvas file. Saved level that places a UI Canvas on a single mesh or multiple meshes when entering Game Mode.

**Suggested Time Box:**
10 minutes

| Workflow                                                              | Requests                                                                                                                                                                                                                                                                                                                                      | Things to Watch For                                                                                    |
|-----------------------------------------------------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------|
| Launch UI Editor                                                      |                                                                                                                                                                                                                                                                                                                                               |                                                                                                        |
| Create and configure new canvas                                       | *   Add various interactive elements<br>*   Configure canvas properties<br>    *   Toggle on Render to texture<br>    *   Select Render Target folder and pick an .attimage                                                                                                                                                                   | *   Rendering issues with visual/text elements                                                         |
| Save canvas                                                           | *   Save and reload the canvas                                                                                                                                                                                                                                                                                                                |                                                                                                        |
| Place UI Canvas on mesh in level                                      | *   Create new entity<br>*   Add the following components:  <br>    *   UI Canvas Asset Ref component (Required)<br>        *   Select newly created canvas<br>        *   Toggle on Load Automatically<br>    *   Mesh<br>    *   Material<br>        *   Model Materials: Edit Material Instance>Base Color>Texture to match the .attiimage | *   Rendering issues with visual/text elements<br>*   Tool display issues/unusable portions of tool    |
| Display interactive UI canvas on entity in level                      | *   Add UI Canvas on Mesh component (Required)<br>    *   Leave blank                                                                                                                                                                                                                                                                         | *   Rendering issues with visual/text elements<br>*   Tool display issues/unusable portions of tool    |
| Display instance of UI Canvas on multiple entities in level           | *   Add UI Canvas Proxy Ref component (Required)<br>    *   Select entity in Viewport<br>*   Create additonal multiple entities                                                                                                                                                                                                               | *   Rendering issues with visual/text elements<br>*   Tool display issues/unusable portions of tool    |
| Display multiple instances of UI Canvas on multiple entities in level | *   Add UI Canvas on Mesh component (Required)<br>    *   Select Render Target Override using .attimage different than canvas<br>*   Create additonal multiple entities                                                                                                                                                                       | *   Rendering issues with visual/text elements<br>*   Tool display issues/unusable portions of tool    |
| Save level                                                            |                                                                                                                                                                                                                                                                                                                                               | *   Asset Processor errors with level processing                                                       |
| Run Game                                                              | *   Press Ctrl + G to run game in the editor<br>*   Run the level in the game launcher                                                                                                                                                                                                                                                        | *   Rendering issues with visual/text elements<br>*   Interactivity issues (Buttons, Drop-downs, etc.) |
